#!/bin/bash

mkdir /tmp/tmpfs
sudo mount -t tmpfs -o size=8G tmpfs /tmp/tmpfs

cd /tmp/tmpfs
git clone git://git.postgresql.org/git/postgresql.git

cd postgresql
./configure -q --enable-debug --enable-cassert --enable-tap-tests --with-liburing && make -s -j8 && TESTS=test_setup make -s check-tests

PGROOT=`pwd`/tmp_install/usr/local/pgsql
PGDATA=`pwd`/tmpdb
export PGDATA 

export PATH="$PGROOT/bin:$PATH"
export LD_LIBRARY_PATH="$PGROOT/lib"

export LANG=en_US.utf8

initdb >initdb.log 2>&1

export PGPORT=15432

echo "
port=$PGPORT
lc_messages = 'C'
fsync = off
autovacuum = off
restart_after_crash = off

checkpoint_timeout = 30s

io_max_concurrency = 10
io_method = io_uring
" > $PGDATA/postgresql.auto.conf

ulimit -c unlimited
ulimit -n 4096

pg_ctl --l server.log start

res=0

for i in `seq 300`; do
  numjobs=$((20 + $RANDOM % 60))
  echo "iteration $i (jobs: $numjobs)"
  date
  for ((j=1;j<=numjobs;j++)); do
    (
      createdb db$j;
      for ((n=1;n<=50;n++)); do
        cat << EOF | psql -d db$j -a >>/dev/null 2>&1
DROP TABLE IF EXISTS tenk1;
CREATE TABLE tenk1 (
    unique1     int4,
    unique2     int4,
    two         int4,
    four        int4,
    ten         int4,
    twenty      int4,
    hundred     int4,
    thousand    int4,
    twothousand int4,
    fivethous   int4,
    tenthous    int4,
    odd         int4,
    even        int4,
    stringu1    name,
    stringu2    name,
    string4     name
);
COPY tenk1 FROM '/tmp/tmpfs/postgresql/src/test/regress/data/tenk.data';
EOF
      done;
    ) &
  done
  wait
  
  for ((j=1;j<=numjobs;j++)); do dropdb db$j & done
  wait
  grep -A3 -E '(ERROR|could not read blocks )' server.log && break;
done

pg_ctl stop

